/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/

package de.hendriklipka.databus;

import java.util.*;
import java.io.*;

import org.w3c.dom.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;

import org.log4j.*;
import org.log4j.xml.*;

public class DataBus extends java.lang.Object
{
    static Category CAT = Category.getInstance(DataBus.class.getName());

    protected static Hashtable _descriptorsByName=new Hashtable();
    protected static Hashtable _descriptorsByClass=new Hashtable();

    protected String _name="unknown";
    protected Hashtable _allConsumer=new Hashtable();
    protected Hashtable _allProducers=new Hashtable();
    protected Hashtable _allComponents=new Hashtable();

    protected Vector _comps=new Vector(5,5);
    protected Vector _gens=new Vector(5,5);

    protected long _eventsInQueue=0;
    protected long _eventsInWork=0;
    protected Object _eiwLock=new Object();
    protected boolean _doReConfig=false;
    protected Object _sendSyncObj=new Object();

    protected Hashtable _listenerTypes=new Hashtable();
    protected Hashtable _producerTypes=new Hashtable();

    public DataBus(Node configNode) throws Exception
    {
        NamedNodeMap nnl=configNode.getAttributes();
        Node attr=nnl.getNamedItem("name");
        _name=attr.getNodeValue();
        NodeList nl=configNode.getChildNodes();
        for (int i=0;i<nl.getLength();i++)
        {
            Node node=nl.item(i);
            if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("component")))
            {
                String cname=null;
                nnl=node.getAttributes();
                attr=nnl.getNamedItem("class");
                if (null==attr)
                {
                    attr=nnl.getNamedItem("name");
                    cname=getClassForComponent(attr.getNodeValue());
                    CAT.debug("got class "+cname+" from component descriptor "+attr.getNodeValue());
                }
                else
                {
                    cname=attr.getNodeValue();
                }
                if (null==cname)
                {
                    throw new Exception("<component> w/o class && w/o name!");
                }
                attr=nnl.getNamedItem("id");
                if (null==attr)
                {
                    throw new Exception("<component> w/o id!");
                }
                CAT.debug("trying to load class: "+cname);
                String id=attr.getNodeValue();
                Class c=Class.forName(cname);
                Object o=c.newInstance();
                DataBusComponent dbc=(DataBusComponent)o;
                ComponentDescriptor cd=getDescriptorForClass(cname);
                ComponentInstance ci=new ComponentInstance();
                ci._id=id;
                ci._component=dbc;
                ci._configNode=node;
                ci._descriptor=cd;
                _allComponents.put(dbc,ci);
                attr=nnl.getNamedItem("position");
                if (null!=attr)
                {
                    ci._position=Integer.parseInt(attr.getNodeValue());
                }
                _comps.addElement(ci);
                CAT.info("added "+cname);
                if (dbc instanceof DataBusProducer)
                {
                    CAT.info("add as producer");
                    _allProducers.put(dbc,cd);
                    if (cd._isGenericProducer)
                    {
                        _gens.addElement(ci);
                    }
                    String[] pta=cd._produces;
                    if (null!=pta)
                    {
                        for (int j=0;j<pta.length;j++)
                        {
                            _producerTypes.put(pta[j],pta[j]);
                        }
                    }
                }
                if (dbc instanceof DataBusConsumer)
                {
                    CAT.info("add as consument");
                    String[] lta=cd._listenTo;
                    if (null!=lta)
                    {
                        for (int j=0;j<lta.length;j++)
                        {
                            _listenerTypes.put(lta[j],lta[j]);
                        }
                    }
                }
            }
        }
        // sort components according to their position
        Collections.sort(_comps);
    }

    public void go() throws Exception
    {
        // initialization
        CAT.debug("init");
        for (int i=0;i<_comps.size();i++)
        {
            ComponentInstance ci=(ComponentInstance)_comps.elementAt(i);
            CAT.info("position for "+ci._id+"="+ci._position);
            ci._component.init(this,ci._configNode);
        }
        // pre startup run
        CAT.debug("pre startup");
        for (int i=0;i<_comps.size();i++)
        {
            ComponentInstance ci=(ComponentInstance)_comps.elementAt(i);
            ci._component.preStartup();
        }
        // startup
        CAT.debug("startup");
        for (int i=0;i<_comps.size();i++)
        {
            ComponentInstance ci=(ComponentInstance)_comps.elementAt(i);
            ci._component.start();
        }
        //post startup run
        CAT.debug("post startup");
        for (int i=0;i<_comps.size();i++)
        {
            ComponentInstance ci=(ComponentInstance)_comps.elementAt(i);
            ci._component.postStartup();
        }
    }

    public String getProperty(String key, String def)
    {
        return _props.getProperty(key,def);
    }

    public String getProperty(String key)
    {
        return getProperty(key,null);
    }

    /**
        insert a new consumer into the bus, for a specific event
        @param dataType name of the event this consumer wants to receive events for (max. 15 chars)
    */
    public void addConsumer(DataBusConsumer dbc, String dataType)
    {
        if (dataType.length()>15)
        dataType=dataType.substring(0,15);
        Vector v=(Vector)_allConsumer.get(dataType);
        if (null==v)
        {
            v=new Vector(3,3);
            _allConsumer.put(dataType,v);
        }
        Consumer c=new Consumer(dbc);
        v.addElement(c);
        c._name=dbc.toString();
        new Thread(c).start();
    }

    /**
        removes a consumer
    */
    public synchronized void removeConsumer(DataBusConsumer dbc)
    {
        Enumeration e=_allConsumer.elements();
        while (e.hasMoreElements())
        {
            Vector v=(Vector)e.nextElement();
            for (int i=0;i<v.size();i++)
            {
                Consumer c=(Consumer)v.elementAt(i);
                if (dbc==c._dbc)
                {
                    v.removeElement(c);
                }
            }
        }
    }

    /**
        removes a component from the bus
    */
    public void componentFinished(DataBusComponent dbc)
    {
        _gens.removeElement(dbc);
        _comps.removeElement(dbc);
        _allComponents.remove(dbc);
        // look if there are generic producers left
        if (0==_gens.size())
        {
            // no
            while(true)
            {
                // wait until there are no more events on the bus
                synchronized(_eiwLock)
                {
                    if ((0==_eventsInQueue)&&(0==_eventsInWork))
                    {
                        break;
                    }
                    try
                    {
                        _eiwLock.wait();
                    }
                    catch (InterruptedException ex)
                    {
                        ex.printStackTrace();
                    }
                }
            }
            Enumeration e=_allConsumer.elements();
            while (e.hasMoreElements())
            {
                Vector v=(Vector)e.nextElement();
                for (int i=0;i<v.size();i++)
                {
                    Consumer c=(Consumer)v.elementAt(i);
                    c._stop=true;
                    synchronized(c._flag)
                    {
                        c._flag.notifyAll();
                    }
                }
            }
            for (int i=0;i<_comps.size();i++)
            {
                ComponentInstance ci=(ComponentInstance)_comps.elementAt(i);
                DataBusComponent dbc2=(DataBusComponent)(ci._component);
                try
                {
                    dbc2.shutdown();
                }
                catch (Exception ex)
                {
                    CAT.error("error during shutdown: ",ex);
                }
            }
        }
    }

    //ToDo
    protected void forceShutDown()
    {
    }

    /**
        distributes events on the bus
        @param data data to be send
        @param dataType type of the event
    */
    public void sendData(Object data,String dataType, DataBusProducer src)
    {
        synchronized(_sendSyncObj)
        {
            // during a reconfiguration we do not accept new data
            if (_doReConfig)
            {
                if (getDescriptor(src)._isGenericProducer)
                {
                    try
                    {
                        _sendSyncObj.wait();
                    }
                    catch (InterruptedException ex) {}
                }
            }
            if (dataType.length()>15)
            dataType=dataType.substring(0,15);
            Vector v=(Vector)_allConsumer.get(dataType);
            if (null==v)
            {
                return;
            }
            DataBusEvent dbe=new DataBusEvent();
            dbe._data=data;
            for (int i=0;i<v.size();i++)
            {
                ((Consumer)v.elementAt(i)).dataAvailable(dbe);
            }
        }
    }

    /**
        distributes data on the bus, but wait for an answer written to the event
        @param data data to be send
        @param dataType type of the event
        @param src source which waits for the answer
        @return the response object
    */
    public Object sendResponseData(Object data,String dataType, DataBusProducer src)
    {
        synchronized(_sendSyncObj)
        {
            // during a reconfiguration we do not accept new data
            if (_doReConfig)
            {
                if (getDescriptor(src)._isGenericProducer)
                {
                    try
                    {
                        _sendSyncObj.wait();
                    }
                    catch (InterruptedException ex) {}
                }
            }
            if (dataType.length()>15)
                dataType=dataType.substring(0,15);
            Vector v=(Vector)_allConsumer.get(dataType);
            if (null==v)
            {
                return null;
            }
            DataBusEvent dbe=new DataBusEvent();
            dbe._data=data;
            dbe._waitForResponse=true;
            for (int i=0;i<v.size();i++)
            {
                ((Consumer)v.elementAt(i)).dataAvailable(dbe);
            }
            // wait until <B>waitForResponse</B> has been reset
            // notification happans through dbe._flag
            synchronized(dbe._flag)
            {
                while(dbe._waitForResponse)
                {
                    try
                    {
                        dbe._flag.wait();
                    }
                    catch (InterruptedException ex)
                    {
                        ex.printStackTrace();
                    }
                }
            }
            return dbe._answer;
        }
    }

    protected void doReConfiguration(Node configNode)
    {
        /*
        CAT.debug("reconfiguring bus...");
        // lock bus
        synchronized(_sendSyncObj)
        {
            _doReConfig=true;
        }
        CAT.debug("waiting until bus has stopped...");
        while(true)
        {
            // wait until there are no more events
            synchronized(_eiwLock)
            {
                if ((0==_eventsInQueue)&&(0==_eventsInWork))
                {
                    break;
                }
                try
                {
                    _eiwLock.wait();
                }
                catch (InterruptedException ex)
                {
                    CAT.error("error during reconfiguration",ex);
                }
            }
        }

        try
        {
            NodeList nl=configNode.getChildNodes();
            for (int i=0;i<nl.getLength();i++)
            {
                Node node=nl.item(i);
                if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("component")))
                {
                    NamedNodeMap nnl=node.getAttributes();
                    Node attr=nnl.getNamedItem("id");
                    if (null==attr)
                    {
                        throw new Exception("<component> w/o id!");
                    }
                    // is this ID already there?
                    // yes -> reconfig
                    // no -> create
                    attr=nnl.getNamedItem("class");
                    if (null==attr)
                    {
                        throw new Exception("<component> w/o class!");
                    }
                    String cname=attr.getNodeValue();
                    String id=attr.getNodeValue();
                    Class c=Class.forName(cname);
                    Object o=c.newInstance();
                    DataBusComponent dbc=(DataBusComponent)o;
                    dbc.setID(id);
                    _comps.addElement(dbc);
                    dbc.init(this,node);
                    if (dbc instanceof DataBusProducer)
                    {
                        if (((DataBusProducer)dbc).isGenericProducer())
                        {
                            _gens.addElement(dbc);
                        }
                        String[] pta=((DataBusProducer)dbc).produces();
                        if (null!=pta)
                        {
                            for (int j=0;j<pta.length;j++)
                            {
                                _producerTypes.put(pta[j],pta[j]);
                            }
                        }
                    }
                    if (dbc instanceof DataBusConsumer)
                    {
                        String[] lta=((DataBusConsumer)dbc).listenTo();
                        if (null!=lta)
                        {
                            for (int j=0;j<lta.length;j++)
                            {
                                _listenerTypes.put(lta[j],lta[j]);
                            }
                        }
                    }
                }
            }
        }
        catch (Exception ex)
        {
            CAT.error("error during reconfiguration",ex);
        }

        CAT.debug("bus stopped");

        // unlock bus
        synchronized(_sendSyncObj)
        {
            _sendSyncObj.notifyAll();
        }
        CAT.debug("bus restarted");
        */
    }

    /**
        this class decouples a consumer from the bus, so that the bus is not blocked until the
        incoming events has been processed
        It uses an own thread, whichc stores the incoming events in an queue and sends them
        to the consumer. If the queue gets too long, the priority of this thread (the processing
        happans in its context) is increased (to have faster processing), until the queue gets an acceptable size.
    */
    protected class Consumer implements Runnable
    {
        String _name="";
        Vector _allEvents=new Vector(5,5);
        Object _flag=new Object();
        DataBusConsumer _dbc=null;
        boolean _stop=false;

        Consumer (DataBusConsumer dbc)
        {
            _dbc=dbc;
        }

        public void run()
        {
            boolean highPri=false;
            while (!_stop)
            {
                DataBusEvent dbe=null;
                synchronized(_flag)
                {
                    if (0!=_allEvents.size())
                    {
                        synchronized(_allEvents)
                        {
                            dbe=(DataBusEvent)_allEvents.elementAt(0);
                            _allEvents.removeElementAt(0);
                            _eventsInQueue--;
                        }
                    }
                }
                if (null!=dbe)
                {
                    try
                    {
                        synchronized (_eiwLock)
                        {
                            _eventsInWork++;
                        }
                        _dbc.dataAvailable(dbe);
                        synchronized (_eiwLock)
                        {
                            _eventsInWork--;
                            _eiwLock.notifyAll();
                        }
                        if (_allEvents.size()<10)
                        Thread.currentThread().yield();
                        //ToDo: increase priority more than once, if neccessary
                        try
                        {
                            // if the queue is too long, increase priority
                            if ((_allEvents.size()>100)&&(!highPri))
                            {
                                highPri=true;
                                Thread t=Thread.currentThread();
                                int i=t.getPriority();
                                t.setPriority(i+1);
                            }
                            // if we have increased priority and the queue is short enough,
                            // reduce priority
                            if ((highPri)&&(_allEvents.size()<10))
                            {
                                highPri=false;
                                Thread t=Thread.currentThread();
                                int i=t.getPriority();
                                t.setPriority(i-1);
                            }
                        }
                        catch (Exception ex)
                        {
                            ex.printStackTrace();
                        }
                    }
                    catch (Throwable ex)
                    {
                        CAT.error("error during data delivery: ",ex);
                    }
                    if (dbe._waitForResponse)
                    {
                        synchronized(dbe._flag)
                        {
                            dbe._waitForResponse=false;
                            dbe._flag.notifyAll();
                        }
                    }
                }
                else
                {
                    synchronized(_flag)
                    {
                        try
                        {
                            _flag.wait();
                        }
                        catch (InterruptedException ex)
                        {
                            CAT.error("got interrupted during waiting for answer",ex);
                        }
                    }
                }
            }
        }

        void dataAvailable(DataBusEvent dbe)
        {
            synchronized(_flag)
            {
                _allEvents.addElement(dbe);
                _eventsInQueue++;
                _flag.notifyAll();
            }
        }
    }

    protected ComponentDescriptor getDescriptorForClass(String classname)
    {
        CAT.debug("get descriptor for class "+classname);
        Object o=_descriptorsByClass.get(classname);
        if (null==o)
        {
            CAT.warn("no descriptor found for class "+classname);
        }
        return (ComponentDescriptor)o;
    }

    protected ComponentDescriptor getDescriptorForName(String name)
    {
        CAT.debug("get descriptor for name "+name);
        return (ComponentDescriptor)_descriptorsByName.get(name);
    }

    protected static void registerComponent(Node node)
    {
        ComponentDescriptor cd=new ComponentDescriptor(node);
        CAT.info("registering component <"+cd._componentName+">, class <"+cd._componentClass+">");
        _descriptorsByClass.put(cd._componentClass,cd);
        _descriptorsByName.put(cd._componentName,cd);
    }

    protected ComponentDescriptor getDescriptor(DataBusProducer comp)
    {
        return (ComponentDescriptor)_allProducers.get(comp);
    }

    protected String getClassForComponent(String name)
    {
        CAT.debug("get class for descriptor "+name);
        return ((ComponentDescriptor)_descriptorsByName.get(name))._componentClass;
    }

    protected ComponentInstance getInstanceForComponent(DataBusComponent dbc)
    {
        return (ComponentInstance)_allComponents.get(dbc);
    }

//-----------------------------------------------------------------------------
// static attributes, classes and methods

    /**
        waiting for a modified config file
    */
    protected static class ConfigChangeThread extends Thread
    {
        protected Object _waitObject=new Object();
        protected String _fileName=null;
        protected long _fileDate=0;
        protected long _sleepTime=15*1000;

        public ConfigChangeThread(String fileName, int sleepTime)
        {
            _fileName=fileName;
            _sleepTime=sleepTime;
            setDaemon(true);
            File f=new File(_fileName);
            _fileDate=f.lastModified();
        }

        public void run()
        {
            CAT.info("config file watcher started");
            while (true)
            {
                synchronized(_waitObject)
                {
                    try
                    {
                        _waitObject.wait(_sleepTime);
                    }
                    catch (InterruptedException ex) {}
                }
                File f=new File(_fileName);
                long d=f.lastModified();
                if (d>_fileDate)
                {
                    // execute reconfiguration
                    CAT.info("detected new config file");
                    _fileDate=d;
                    doReConfiguration(_fileName);
                }
            }
        }
    }

    // reconfig all databusses
    protected static void doReConfiguration(String fileName)
    {
        Vector newBusses=new Vector(5,5);
        try
        {
            // open config file
            InputSource input = Resolver.createInputSource(new File(fileName));
            XmlDocument doc = XmlDocument.createXmlDocument(input, false);
            Element el=doc.getDocumentElement();

            /*
            // we don't reconfig log4j because then the logfile is closed and re-opened :(
            NodeList cnl=el.getElementsByTagName("log4j");
            Node configNode=cnl.item(0);
            DOMConfigurator.configure((Element)configNode);
            */
            
            // read all busses
            NodeList nl=el.getElementsByTagName("databus");
            for (int i=0;i<nl.getLength();i++)
            {
                Node node=nl.item(i);
                NamedNodeMap nnl=node.getAttributes();
                Node attr=nnl.getNamedItem("name");
                String name=attr.getNodeValue();
                // do we already have this bus ?
                DataBus db=null;
                for (int j=0;j<_busses.size();j++)
                {
                    DataBus db1=(DataBus)_busses.elementAt(j);
                    if (name.equals(db1._name))
                    {
                        db=db1;
                        break;
                    }
                }
                if (null!=db)
                {
                    // yes
                    db.doReConfiguration(node);
                }
                else
                {
                    // no -> create new bus
                    db=new DataBus(node);
                    _busses.addElement(db);
                    db.go();
                }
                newBusses.addElement(name);
            }
            // look for existing busses which are not in the file anymore
            for (int i=0;i<_busses.size();i++)
            {
                String name1=((DataBus)_busses.elementAt(i))._name;
                boolean found=false;
                for (int j=0;j<newBusses.size();j++)
                {
                    String name2=(String)newBusses.elementAt(i);
                    if (name2.equals(name1))
                    {
                        found=true;
                        break;
                    }
                }
                if (!found)
                {
                    // stop this bus
                    DataBus db=(DataBus)_busses.elementAt(i);
                    db.forceShutDown();
                    _busses.removeElement(db);
                }
            }
        }
        catch (Exception ex)
        {
            CAT.error("error during reconfiguration :",ex);
        }
    }

    protected static Properties _props=new Properties();
    protected static Vector _busses=new Vector(5,5);

    public static void main(String[] args)
    {
        try
        {
            String fName="config.xml";
            if ((null!=args)&&(0!=args.length))
            {
                fName=args[0];
                if (1<args.length)
                {
                    for (int i=1;i<args.length;i++)
                    {
                        try
                        {
                            String s=args[i];
                            StringTokenizer st=new StringTokenizer(s,"=");
                            String name=(String)st.nextElement();
                            String val=(String)st.nextElement();
                            CAT.info("Property: "+name+"="+val);
                            _props.put(name,val);
                        }
                        catch (Exception ex)
                        {
                            System.err.println("error while parsing command line properties: no '=' found in parameter, use \"");
                            System.exit(1);
                        }
                    }
                }
            }

            InputSource input = Resolver.createInputSource(new File(fName));
            XmlDocument doc = XmlDocument.createXmlDocument(input, false);
            Element el=doc.getDocumentElement();

            NodeList cnl=el.getElementsByTagName("log4j");
            Node configNode=cnl.item(0);
            DOMConfigurator.configure((Element)configNode);

            // register components from external file
            String cfName=System.getProperty("databus.components");
            if (null==cfName)
            {
                cfName="components.xml";
            }

            InputSource cinput = Resolver.createInputSource(new File(cfName));
            XmlDocument cdoc = XmlDocument.createXmlDocument(cinput, false);
            Element cel=cdoc.getDocumentElement();
            NodeList nl=cel.getElementsByTagName("component");
            for (int i=0;i<nl.getLength();i++)
            {
                Node node=nl.item(i);
                registerComponent(node);
            }

            // after that, create all busses
            nl=el.getElementsByTagName("databus");
            for (int i=0;i<nl.getLength();i++)
            {
                Node node=nl.item(i);
                DataBus db=new DataBus(node);
                _busses.addElement(db);
            }
            for (int i=0;i<_busses.size();i++)
            {
                DataBus db=(DataBus)(_busses.elementAt(i));
                db.go();
            }
            ConfigChangeThread cct=new ConfigChangeThread(fName,10000);
            cct.start();
        }
        catch (Throwable ex)
        {
            CAT.error("error during init:",ex);
            System.exit(1);
        }
    }

    public static String getTextData(Node node)
    {
        NodeList subs=node.getChildNodes();
        String s="";
        for (int j=0;j<subs.getLength();j++)
        {
            Node attr=subs.item(j);
            if ((attr.getNodeType()!=attr.TEXT_NODE)&&(attr.CDATA_SECTION_NODE!=attr.getNodeType()))
                continue;
            Text val=(Text)attr;
            s+=val.getData().trim();
        }
        if (0==s.length())
            return null;
        return s;
    }
}
