package de.hendriklipka.gcp.module.admin.handler;

import java.util.*;
import java.io.*;

import org.log4j.Category;

import org.w3c.dom.*;

import koala.dynamicjava.interpreter.*;
import koala.dynamicjava.parser.wrapper.*;
       
import de.hendriklipka.databus.*;
import de.hendriklipka.util.*;

import de.hendriklipka.gcp.exception.*;
import de.hendriklipka.gcp.event.*;

/**
    a command handler which reads the code for the commands from the config file
    and executes them via DynamicJava
    @see de.hendriklipka.gcp.module.input.network.NetworkControlHandler
*/
public class DynamicCommandHandler extends de.hendriklipka.gcp.module.admin.handler.CommandHandler
{
    static Category CAT = Category.getInstance(CommandHandler.class.getName());

    protected Hashtable _commands=new Hashtable();
    protected Handler _handler=new Handler();
    
    protected String[] _commandNames=null;
    
    public void init(DataBus bus, Node configNode)
    {
        super.init(bus,configNode);

        Vector v=new Vector();
        NodeList nl=configNode.getChildNodes();
        for (int i=0;i<nl.getLength();i++)
        {
            Node node=nl.item(i);
            if (node.getNodeType()==node.ELEMENT_NODE)
            {
                if(node.getNodeName().equals("command"))
                {
                    NamedNodeMap attrs=node.getAttributes();
                    Node attr=attrs.getNamedItem("name");
                    String name=attr.getNodeValue();
                    
                    String command=XMLUtil.getTextData(node)+"\n";

                    _commands.put(name,command);
                    v.addElement(name);
                }
            }
        }
        _commandNames=new String[v.size()];
        for (int i=0;i<v.size();i++)
        {
            _commandNames[i]=(String)v.elementAt(i);
        }
    }
            
    public String[] getHandlerCommands()
    {
        return _commandNames;
    }

    public CommandResponse handleCommand(String s)
    { 
        CAT.info("handling: "+s);
        String cmd=s.trim();
        String param="";
        int pos=cmd.indexOf(' ');
        if (-1!=pos)
        {
            cmd=cmd.substring(0,pos-1);
            param=cmd.substring(pos+1);
        }
        Object o=_commands.get(cmd);
        if (null!=o)
        {
            String cmds=(String)o;
            StringReader sis=new StringReader(cmds);
            
            // create new interpreter
            Interpreter interpreter = new TreeInterpreter(new JavaCCParserFactory());
            
            String result="--";

            // put variables into the interpreter            
            interpreter.defineVariable("handler", _handler);
            interpreter.defineVariable("result", result);
                           
            try 
            {
                // execute command
                interpreter.interpret(sis,"djava");
                result=(String)interpreter.getVariable("result");
            } 
            catch (Throwable ex) 
            {
                CAT.error("while executing dynamic command: ",ex);
                return new CommandResponse(ex.getMessage());
            }            
            return new CommandResponse(result);
        }
        return new CommandResponse();
    }
    
    public class Handler
    {
        public String sendCommand(String cmd)
        {
            return cmd+" - ok";
        }
    }
}
