/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.control;

import java.util.*;

import org.w3c.dom.Node;
import org.log4j.Category;

import de.hendriklipka.databus.*;

import de.hendriklipka.gcp.module.*;
import de.hendriklipka.gcp.exception.*;
import de.hendriklipka.gcp.event.*;

/**
    sends info requests or commands to running game servers
*/
public abstract class GameCommandHandler implements DataBusComponent, DataBusConsumer
{
    static Category CAT = Category.getInstance(GameCommandHandler.class.getName());

    protected DataBus _bus=null;

    protected String _error=null;

    public void init(DataBus bus, Node configNode) throws Exception
    {
        _bus=bus;
        _bus.addConsumer(this,GameControlEvent.NAME);
    }

    abstract public GameControlResponse handleEvent(GameControlEvent event);

    public void dataAvailable(de.hendriklipka.databus.DataBusEvent event)
    {
        try
        {
            GameControlEvent gce=(GameControlEvent)event._data;
            GameControlResponse r=handleEvent(gce);
            event._answer=r;
        }
        catch (Exception ex)
        {
            CAT.error(ex);
        }
    }

    public void start()
    {
    }

    public void shutdown()
    {
    }

    public void deInitialize()
    {
    }

    public void reconfig(Node configNode)
    {
    }

    public abstract String sendRequest(String command) throws HandleException; // USER_DEFINED
    public abstract void sendShutdownRequest() throws HandleException;; // SHUTDOWN
    public abstract ServerInfo sendServerStatusRequest() throws HandleException;; // SERVER_STATUS
    public abstract PlayerInfo[] sendPlayerListRequest() throws HandleException;; // PLAYER_LIST
    public abstract PlayerInfo sendPlayerInfoRequest(String name) throws HandleException;; // PLAYER_INFO_FOR_NAME
    public abstract PlayerInfo sendPlayerInfoRequest(long id) throws HandleException;; // PLAYER_INFO_FOR_ID
    public abstract void sendSetLocalVarRequest(String name, String value) throws HandleException;; // SET_LOCAL_VARS
    public abstract Properties sendGetLocalVarsRequest() throws HandleException;; // GET_LOCAL_VARS
    public abstract void sendSetServerVarRequest(String name, String value) throws HandleException;; // SET_SERVER_VARS
    public abstract Properties sendGetServerVarsRequest() throws HandleException;; // GET_SERVER_VARS
    public abstract void sendMapChangeRequest(String name) throws HandleException;; // MAP_CHANGE
    public abstract void sendKickUserRequest(long id) throws HandleException;; // KICK_USER
    public abstract void sendMessageRequest(String msg) throws HandleException;; // MESSAGE
    public abstract void sendMessageToUserRequest(long id, String msg) throws HandleException;;

    public void preStartup() throws Exception
    {
    }

    public void postStartup() throws Exception
    {
    }
}
