/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.control;

import java.util.Properties;
import java.util.Enumeration;
import java.util.Vector;

import org.log4j.Category;

/**
    this class encapsulates information about a server
*/
public class ServerInfo
{
    static Category CAT = Category.getInstance(ServerInfo.class.getName());

  public void setData(Properties props)
  {
    if (null==_data)
      _data=props;
    else
    {
      Enumeration e=props.propertyNames();
      while (e.hasMoreElements())
      {
        String k=(String)e.nextElement();
        _data.put(k,props.getProperty(k));
      }
    }
  }
  
  public void copy(ServerInfo si)
  {
    if (null!=si._data)
    {
      setData(si._data);
    }
    
    // get player information
    if(si._players!=null)
    {
      if (null==_players)
      {
        _players=si._players; 
      }
      else
      {
        Vector v=new Vector();
        // step through all players in si
        for (int i=0;i<si._players.length;i++)
        {
          boolean found=false;
          // already there?
          for (int j=0;j<_players.length;j++)
          {
            // if identical, combine
            if (_players[j]._id.equals(si._players[i]._id))
            {
              found=true;
              _players[j].copy(si._players[i]);
            }
          }
          // if new, add to list
          if (!found)
          {
            v.addElement(si._players[i]);
          }
        }
        if (0!=v.size())
        {
          PlayerInfo[] pia=new PlayerInfo[_players.length+v.size()];
          System.arraycopy(_players,0,pia,0,_players.length);
          for (int i=0;i<v.size();i++)
          {
            pia[i+_players.length]=(PlayerInfo)v.elementAt(i);
          }
          _players=pia;
        }
      }
    }
  }
  
  public PlayerInfo[] _players=null;
  
  public void set(String name, String value)
  {
    _data.put(name,value);
  }
  
  public Properties getData()
  {
    return _data;
  }
  
  public String toString()
  {
    String s="";
    Enumeration e=_data.keys();
    while (e.hasMoreElements())
    {
        String n=(String)e.nextElement();
        String v=_data.getProperty(n);
        s+=n+"="+v+"\r\n";
    }
    return s;
  }
  
  protected Properties _data=new Properties();
}
