/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.control.quake;

import java.net.*;
import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import org.log4j.Category;

import de.hendriklipka.databus.*;

import de.hendriklipka.util.*;

import de.hendriklipka.gcp.exception.*;
import de.hendriklipka.gcp.event.*;
import de.hendriklipka.gcp.module.control.*;

/**
  sends requests to a running Quake3Arena-Server via UDP.
*/
public class Q3AProtocolHandler extends QWProtocolHandler
{
    static Category CAT = Category.getInstance(Q3AProtocolHandler.class.getName());

    public String sendARequest(String command) throws HandleException
    {
        CAT.debug("request: "+command);
        String result=null;
        try
        {
            byte[] p=sendPacket(command,_ip,_port);
            int len=p.length;
            StringBuffer sb=new StringBuffer();
            for (int i=0;i<len;i++)
            sb.append((char)p[i]);
            result=resultToString(sb.toString());
            if (result.equals("Bad rconpassword."))
            {
                throw new HandleException("Bad rcon_password");
            }
        }
        catch (HandleException ex) 
        {
            throw ex;
        }
        catch (Exception ex) 
        {
            throw new HandleException(ex);
        }
        if (null==result)
        {
            throw new HandleException("no result received within timeout");
        }
        return result;
    }

    public ServerInfo sendServerStatusRequest() throws HandleException
    {
        String s=sendARequest("getstatus");
        ServerInfo si=parseServerInfo1(s);
        s=sendARequest("rcon "+_password+" status");
        ServerInfo si2=parseServerInfo2(s);
        s=sendARequest("rcon "+_password+" serverinfo");
        ServerInfo si3=parseServerInfo3(s,"Server info settings:");
        s=sendARequest("rcon "+_password+" systeminfo");
        ServerInfo si4=parseServerInfo3(s,"System info settings:");
        si.copy(si2);
        si.copy(si3);
        si.copy(si4);
        return si;
    }
      
    public PlayerInfo[] sendPlayerListRequest() throws HandleException
    {
        String s=sendARequest("getstatus");
        ServerInfo si=parseServerInfo1(s);
        return si._players;
    }
      
    public PlayerInfo sendPlayerInfoRequest(String name) throws HandleException
    {
        String s=sendARequest("status");
        ServerInfo si=parseServerInfo1(s);
        s=sendARequest("rcon "+_password+" status");
        ServerInfo si2=parseServerInfo2(s);
        si.copy(si2);
        if (null!=si._players)
        {
            for (int i=0;i<si._players.length;i++)
            {
                PlayerInfo pi=si._players[i];
                if (name.equals(pi.getName()))
                {
                    s=sendARequest("rcon "+_password+" user "+pi.getID());
                    PlayerInfo pi2=parsePlayerInfo(s);
                    pi.copy(pi2);
                    return pi;
                }
            }
        }
        throw new HandleException("player not found");
    } 
      
    protected String resultToString(String s)
    {
        while ((char)-1==s.charAt(0))
            s=s.substring(1);
        if (s.startsWith("n"))
            s=s.substring(1);
        if (s.startsWith("\\"))
            s=s.substring(1);
        if (s.startsWith("print"))
            s=s.substring(5);
        if (s.startsWith("statusResponse"))
            s=s.substring(14);
        if (s.startsWith("\n"))
            s=s.substring(1);
        return s;
    }

    public PlayerInfo sendPlayerInfoRequest(long id) throws HandleException
    {
        String sid=Long.toString(id);
        String s=sendARequest("status");
        ServerInfo si=parseServerInfo1(s);
        s=sendARequest("rcon "+_password+" status");
        ServerInfo si2=parseServerInfo2(s);
        si.copy(si2);
        if (null!=si._players)
        {
            for (int i=0;i<si._players.length;i++)
            {
                PlayerInfo pi=si._players[i];
                if (sid.equals(pi.getID()))
                {
                    s=sendARequest("rcon "+_password+" dumpuser "+pi.getID());
                    PlayerInfo pi2=parsePlayerInfo(s);
                    pi.copy(pi2);
                    return pi;
                }
            }
        }
        throw new HandleException("player not found");
    } 
// ToDo
/*
    public Properties sendGetLocalVarsRequest() throws HandleException
    {
        //ToDo
        String s=sendARequest("rcon "+_password+" serverinfo");
        ServerInfo si3=parseServerInfo3(s,"Server info settings:");
        return si3.getData();
    }
      
    public void sendMessageToUserRequest(long id, String msg) throws HandleException
    {
        throw new HandleException("operation not supported");
    }

*/

    // parse "getstatus"
    /*
    \maxclients\8\spawn\0\watervis\0\*version\2.30\*gamedir\fortress\hostname\TF on w
    anderer\maxspectators\5\maxplayer\10\maxplayers\10\s\off\g\on\deathmatch\3\teamp
    lay\31?TF2.5\timelimit\30\fraglimit\100\samelevel\2\*progs\59258\map\2fort5\n\2
    -1 0 "Keel" // ID qport??? "Name"
    */
    public ServerInfo parseServerInfo1(String s)
    {
        int pos=s.indexOf('\n');
        String pl=null;
        if (-1!=pos)
        {
            pl=s.substring(pos);
            s=s.substring(0,pos);
        }
        Properties p=new Properties();
        if (s.startsWith("\\")) // could be missing
            s=s.substring(1);
        StringTokenizer st=new StringTokenizer(s,"\\");
        while (st.hasMoreElements())
        {
            String name=((String)st.nextElement()).trim();
            String value=((String)st.nextElement()).trim();
            p.put("server_"+name,value);
        }
        ServerInfo si=new ServerInfo();
        si.setData(p);
            
        // parse existing players
        if (null!=pl)
        {
            Vector v=new Vector();
            st=new StringTokenizer(pl,"\n");
            while (st.hasMoreElements())
            {
                String line=(String)st.nextElement();
                line=line.trim();
                if (0==line.length())
                    continue;
                pos=line.indexOf(' ');
                String id=line.substring(0,pos);
                line=line.substring(pos+1);

                pos=line.indexOf(' ');
                String qport=line.substring(0,pos);
                line=line.substring(pos+1);

                if (line.startsWith("\""))
                    line=line.substring(1);
                pos=line.indexOf('"');
                String name=new String(line.substring(0,pos));
                PlayerInfo pi=new PlayerInfo();
                pi.setID(id);
                pi.setName(name);
                pi.set("qport",qport);
                v.addElement(pi);
            }
            if (0!=v.size())
            {
                si._players=new PlayerInfo[v.size()];
                for (int i=0;i<v.size();i++)
                {
                    si._players[i]=(PlayerInfo)v.elementAt(i);
                }
            }
        }
        return si;
    }

    // parse "rcon status" 
    /*
        map: q3dm1
        num score ping name           lastmsg address               qport rate
        --- ----- ---- -------------- ------- --------------------- ----- ----
          0   14     0 Keel                50 bot                       0 16384
    */
    public ServerInfo parseServerInfo2(String s)
    {
        ServerInfo si=new ServerInfo();
        Vector v=new Vector();
        StringTokenizer st=new StringTokenizer(s,"\n");
        String d=(String)st.nextElement(); // map
        String map=d.substring(5);
        si.set("map",map);
        st.nextElement(); // num score....
        st.nextElement(); // ---...
            
        while (st.hasMoreElements())
        {
            try
            {
                boolean isSpec=false;
                boolean connecting=false;
                String line=(String)st.nextElement();
                        
                String id=line.substring(0,3).trim();
                line=line.substring(3);
                while (line.startsWith(" "))
                    line=line.substring(1);
                        
                int pos=line.indexOf(' ');
                String score=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);
                        
                pos=line.indexOf(' ');
                String ping=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);

                pos=line.indexOf(' ');
                String name=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);
                        
                pos=line.indexOf(' ');
                String lastmsg=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);
                    
                pos=line.indexOf(' ');
                String address=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);

                pos=line.indexOf(' ');
                String qport=line.substring(0,pos);
                line=line.substring(pos);
                while (line.startsWith(" "))
                    line=line.substring(1);

                pos=line.indexOf(' ');
                String rate=line.substring(0,pos);
                        
                PlayerInfo pi=new PlayerInfo();
                pi.setName(name);
                pi.setID(id);
                pi.set("frags",score);
                pi.set("address",address);
                pi.set("currentrate",rate);
                pi.set("ping",ping);
                pi.set("qport",qport);
                pi.set("lastmsg",qport);
                v.addElement(pi);
            }
            catch (Exception ex) 
            {
            }
        }
        if (0!=v.size())
        {
            si._players=new PlayerInfo[v.size()];
            for (int i=0;i<v.size();i++)
            {
                si._players[i]=(PlayerInfo)v.elementAt(i);
            }
        }
        return si;
    }

    // parse "dumpuser #"
    public PlayerInfo parsePlayerInfo(String s)
    {
        Properties p=new Properties();
        StringTokenizer st=new StringTokenizer(s,"\n");
        String fl=(String)st.nextElement();
        if (!fl.startsWith("userinfo"))
            return null;
        st.nextElement(); // --------
        while (st.hasMoreElements())
        {
            String line=(String)st.nextElement();
            if (line.length()<=20)
                continue;
            String name=line.substring(0,20).trim();
            String value=line.substring(20).trim();
            p.put(name,value);
        }
        PlayerInfo pi=new PlayerInfo();
        pi.setData(p);
        return pi;
    }
}
