/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.event;

import java.io.*;
import java.util.*;
import java.text.DateFormat;

import org.w3c.dom.Node;
import org.log4j.Category;

import de.hendriklipka.util.*;

import de.hendriklipka.databus.*;

import de.hendriklipka.gcp.event.*;
import de.hendriklipka.gcp.exception.*;

/**
    writes a new log file for every map
    the map change is detected when a GameEvent with a configured name occurs
    the name for the output file can contain the mapname (%m) and the current time (%t). the output
    is appended to existing files
*/
public class ProgramOnEventHandler implements DataBusComponent, DataBusConsumer
{
    static Category CAT = Category.getInstance(ProgramOnEventHandler.class.getName());

    protected DataBus _bus=null;

    protected Options _options=new Options();
    protected DateFormat _format=DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT);
    
    public ProgramOnEventHandler()
    {
    }

    public void init(DataBus bus, Node configNode) throws Exception
    {
        _bus=bus;

        configure(configNode);

        _bus.addConsumer(this,GameEvent.NAME);
    }
    
    public void configure(Node configNode) throws Exception
    {
        reconfig(configNode);
    }

    protected void callProgram(Date time, Properties data)
    {
        String name=_options._programName;
        String s=_format.format(time);
        name=Util.replaceAllSubStrings(name,"%t",s);
        Enumeration e=data.keys();
        while (e.hasMoreElements())
        {
            String pname=(String)e.nextElement();
            String value=data.getProperty(pname);
            name=Util.replaceAllSubStrings(name,"%"+pname+"%",value);
        }
        try
        {
            CAT.debug("calling <"+name+">");
            Runtime.getRuntime().exec(name);
        }
        catch (Throwable ex)
        {
            CAT.error(ex);
        }        
    }
        
    public synchronized void dataAvailable(de.hendriklipka.databus.DataBusEvent o) throws Exception
    {
        try
        {
            GameEvent ge=(GameEvent)(o._data);
            if (ge._type.equals(_options._eventType))
            {
                callProgram(ge._eventDate,ge._data);
            }
        }
        catch (Throwable ex)
        {
            CAT.error(ex);
        }
    }

    public void start() throws Exception
    {
    }
        
    public void shutdown() throws Exception
    {
    }
        
    public void deInitialize() throws Exception
    {
    }
        
    public void reconfig(Node configNode) throws Exception
    {
        // the configuration is not active until the next map change
        String[][] rules={
                        {"_programName","program"},
                        {"_eventType","eventtype"}};
        String[] checkRules={
                        "_programName",
                        "_eventType"};
        XMLUtil.readXMLConfigNode(configNode,rules,checkRules,null,_options);
    }
        
    public void preStartup() throws Exception
    {
    }
    
    public void postStartup() throws Exception
    {
    }
    
    public class Options
    {
        public String _programName=null;
        public String _eventType=null;
    }
}
