/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.event;

import java.net.*;

import org.w3c.dom.*;

import org.log4j.Category;

import de.hendriklipka.databus.*;

import de.hendriklipka.util.*;

import de.hendriklipka.gcp.event.*;

public class QuakeSendCommand implements DataBusComponent, DataBusProducer, Runnable
{
    static Category CAT = Category.getInstance(QuakeSendCommand.class.getName());

    protected boolean _isShutDown=false;
    protected DataBus _bus;

    protected String _ip="127.0.0.1";
    protected int _port=27500;
    protected int _sleepTime=5000;
    protected String _command="getstatus";

    public void init(DataBus bus, Node configNode) throws java.lang.Exception
    {
        _bus=bus;
        readConfig(configNode);
    }

    protected void readConfig (Node configNode)
    {
        NodeList nl=configNode.getChildNodes();
        for (int i=0;i<nl.getLength();i++)
        {
            Node node=nl.item(i);
            if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("address")))
            {
                NamedNodeMap nnl=node.getAttributes();
                Node attr=nnl.getNamedItem("ip");
                if (null!=attr)
                {
                    _ip=attr.getNodeValue();
                }
                attr=nnl.getNamedItem("port");
                if (null!=attr)
                {
                    _port=Integer.parseInt(attr.getNodeValue());
                }
            } // if
            else if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("sleeptime")))
            {
                _sleepTime=1000*Integer.parseInt(XMLUtil.getTextData(node));
            }
            else if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("command")))
            {
                _command=XMLUtil.getTextData(node);
            }
        }
        _isShutDown=false;
    }

    public void run()
    {
        while (!_isShutDown)
        {
            sendCommand();
            synchronized (this)
            {
                try
                {
                    wait(_sleepTime);
                }
                catch (InterruptedException ex)
                { // ignore - we _should_ never be interrupted
                }

            }

        }
    }

    protected void sendCommand()
    {
        DatagramSocket ds=null;
        try
        {
            int cmdLen=_command.length();
            ds=new DatagramSocket(5000);
            byte[] ba=new byte[cmdLen+5];
            byte[] br=new byte[10000];
            ba[0]=(byte)255;
            ba[1]=(byte)255;
            ba[2]=(byte)255;
            ba[3]=(byte)255;
            for (int i=0;i<cmdLen;i++)
            {
                ba[4+i]=(byte)_command.charAt(i);
            }
            ba[cmdLen+4]=0;
            DatagramPacket dp=new DatagramPacket(ba,cmdLen+5,InetAddress.getByName(_ip),_port);
            ds.send(dp);
        }
        catch (Throwable ex)
        {
        }
        finally
        {
            ds.close();
        }
    }

    public void start() throws java.lang.Exception
    {
        Thread t=new Thread(this);
        t.start();
    }

    public void shutdown() throws java.lang.Exception
    {
        _isShutDown=true;
    }

    public void deInitialize() throws java.lang.Exception
    {
        _isShutDown=true;
    }

    public void reconfig(Node configNode) throws java.lang.Exception
    {
        readConfig(configNode);
    }

     public void preStartup() throws Exception
    {
    }
    
    public void postStartup() throws Exception
    {
    }
}