/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.input;

import java.io.*;
import java.util.*;
import java.text.DateFormat;

import org.w3c.dom.Node;
import org.log4j.Category;

import de.hendriklipka.gcp.exception.*;
import de.hendriklipka.gcp.event.InputEvent;
import de.hendriklipka.databus.*;

/**
  reads log files created by QControl. In these files each line is prepended
  by a time stamp.
*/
public class QControlLogFileInput implements DataBusComponent, DataBusProducer, Runnable
{
    static Category CAT = Category.getInstance(QControlLogFileInput.class.getName());

    protected DataBus _bus=null;

    protected BufferedReader InputReader=null;
    protected long LineCount=0;

    public void init(DataBus bus, Node configNode) throws Exception
    {
        // ToDo: converting table?
        _bus=bus;
        CAT.debug("QControlInputFilter.init()");
        String fname=_bus.getProperty("inputfile","clanlog.txt");
        InputReader=new BufferedReader(new InputStreamReader(new FileInputStream(fname)));
    }

  public void run()
  {
    CAT.debug("QControlInputFilter.run()");
    DateFormat df=DateFormat.getInstance();
    try
    {
      while (true==InputReader.ready())
      {
        String s=InputReader.readLine();
        LineCount++;
        String s1=null;
        try
        {
          s1=s.substring(30);
        }
        catch (StringIndexOutOfBoundsException ex)
        {
          continue;
        }
        // filter lines not from the qcontrol console
        if (0!=s.substring(25,29).compareTo("con:"))
        {
          continue;
        }
        // convert hex number ([xx]) into normal chars
        StringBuffer r=new StringBuffer();
        int l=s1.length();
        for (int i=0;i<l;i++)
        {
          char c=s1.charAt(i);
          if ((i<l-3)&&('['==c)&&(']'==s1.charAt(i+3)))
          {
            try
            {
              int j=Integer.parseInt(s1.substring(i+1,i+3),16);
              r.append((char)j);
            }
            catch (NumberFormatException ex) {}
            i+=3;
          }
          else
          {
            r.append(c);
          }
        }
        // check for empty line
        if (0==r.length())
        {
          continue;
        }
        InputEvent ie=new InputEvent();
        ie._line=r.toString();
        try
        {
          ie._eventDate=df.parse(s.substring(0,24));
        }
        catch (java.text.ParseException ex)
        {
          ie._eventDate=new Date();
        }
        _bus.sendData(ie,InputEvent.NAME,this);
      }
      CAT.debug("qcfif finished, exiting");
      _bus.componentFinished(this);
    }
    catch (Exception ex)
    {
      CAT.error(ex);
    }
    return;
  }

    public void start() throws Exception
    {
        CAT.debug("QControlInputFilter.start()");
        new Thread(this).start();
    }

    public void shutdown()
    {
    }

    public void deInitialize()
    {
    }

    public void reconfig(Node configNode)
    {
    }

    public void preStartup() throws Exception
    {
    }

    public void postStartup() throws Exception
    {
    }
}
