/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker;

import java.util.*;

import org.w3c.dom.*;

import org.log4j.Category;

import de.hendriklipka.databus.*;

import de.hendriklipka.util.*;

import de.hendriklipka.gcp.event.*;
import de.hendriklipka.gcp.module.control.PlayerInfo;

/**
    allows players to reserve their nick names
    if a player with a reserved name connects to the server, he needs to
    authenticate with a password. if he cannot authenticate, he is kicked
    authentication is done via rcon commands 
    @see de.hendriklipka.gcp.module.processing.RConCommandFilter
*/
public class IdentKicker implements DataBusComponent, DataBusConsumer, DataBusProducer
{
    static Category CAT = Category.getInstance(IdentKicker.class.getName());

    protected Options _options=null;
    
    protected DataBus _bus=null;

    // Threads, hashed by IP
    protected Hashtable _threads=new Hashtable();
        
    public void init(DataBus bus, Node configNode) throws Exception
    {
        _bus=bus;
        _bus.addConsumer(this,GameEvent.NAME);
        _bus.addConsumer(this,RConUserCommandEvent.NAME);
        
        reconfig(configNode);
    }
        
    public void dataAvailable(DataBusEvent event)
    {
        try
        {
            if (event._data instanceof RConUserCommandEvent)
            {
                RConUserCommandEvent ruce=(RConUserCommandEvent)(event._data);
                handleRConEvent(ruce);
            }
            else if (event._data instanceof GameEvent)
            {
                GameEvent ge=(GameEvent)event._data;
                handleGameEvent(ge);
            }
        }
        catch (Exception ex)
        {
            CAT.error(ex);
        }
    }

    public void handleGameEvent(GameEvent ge)
    {
        String name=handleGameEventAddPlayer(ge);
        if (null==name)
            name=handleGameEventRenamePlayer(ge);
        if (null!=name)
        {
            String pwd=_options._players.getProperty(name);
            if (null==pwd)
            {
                return;
            }
            // determine IP of player
            GameControlEvent gce=new GameControlEvent();
            gce._type=GameControlEvent.PLAYER_INFO_FOR_NAME;
            gce._param1=name;
            GameControlResponse rp=(GameControlResponse)_bus.sendResponseData(gce,GameControlEvent.NAME,this);
            PlayerInfo pi=rp._playerInfo;
            String ip=pi.getData().getProperty(_options._propertyAddress);
            String id=pi.getID();

            PlayerKickThread pkt=new PlayerKickThread(pwd,name,id,_options._tries);
            _threads.put(ip,pkt);
            pkt.start();
        }
    }
    
    protected String handleGameEventAddPlayer(GameEvent ge)
    {
        try
        {
            if (null==_options._addPlayerMsgType)
            {
                return null;
            }
            if (!_options._addPlayerMsgType.equals(ge._type))
            {
                return null;
            }
            if ((null!=_options._addPlayerMsgID)&&(!_options._addPlayerMsgID.equals(ge._id)))
            {
                return null;
            }
            String name=ge._data.getProperty(_options._addPlayerPropertyName);
            return name;
        }
        catch (Exception ex)
        {
            CAT.error(ex);
        }
        return null;
    }
    
    protected String handleGameEventRenamePlayer(GameEvent ge)
    {
        try
        {
            if (null==_options._renamePlayerMsgType)
            {
                return null;
            }
            if (!_options._renamePlayerMsgType.equals(ge._type))
            {
                return null;
            }
            if ((null!=_options._renamePlayerMsgID)&&(!_options._renamePlayerMsgID.equals(ge._id)))
            {
                return null;
            }
            String name=ge._data.getProperty(_options._renamePlayerPropertyName);
            return name;
        }
        catch (Exception ex)
        {
            CAT.error(ex);
        }
        return null;
    }
    
    protected void handleRConEvent(RConUserCommandEvent ruce)
    {
        if (null==ruce._command)
        {
            return;
        }
        if (!ruce._command.startsWith(_options._authCommand))
        {
            return;
        }
        String pwd=ruce._command.substring(_options._authCommand.length()).trim();
        String from=ruce._ip;
        Object o=_threads.get(from);
        if (null!=o)
        {
            PlayerKickThread pkt=(PlayerKickThread)o;
            if (pkt.authenticate(pwd))
            {
                _threads.remove(from);
            }
        }
    }
    
    private class PlayerKickThread extends Thread implements DataBusProducer
    {
        protected String _password=null;
        protected String _id=null;
        protected String _name=null;
        protected Object _syncObj=new Object();
        protected boolean _auth=false;
        protected int _tries=0;
        
        public PlayerKickThread(String pwd, String name, String id, int tries)
        {
            _password=pwd;
            _name=name;
            _id=id;
            _tries=tries;
            GameControlEvent gce=new GameControlEvent();
            gce._type=GameControlEvent.MESSAGE;
            gce._param1=Util.replaceSubString(_options._firstMsg,"%p",name);
            _bus.sendData(gce,GameControlEvent.NAME,this);
        }
        
        public void run()
        {
            while(_tries>0)
            {
                // write message
                GameControlEvent gce=new GameControlEvent();
                gce._type=GameControlEvent.MESSAGE;
                gce._param1=Util.replaceSubString(_options._warnMsg,"%p",_name);
                _bus.sendData(gce,GameControlEvent.NAME,this);
                
                // wait
                synchronized(_syncObj)
                {
                    try
                    {
                        _syncObj.wait(_options._sleepTime);
                    }
                    catch (InterruptedException ex) {}
                }
                _tries--;
                if (_auth)
                    break;
            }
            if (!_auth)
            {
                // kick
                GameControlEvent gce=new GameControlEvent();
                gce._type=GameControlEvent.MESSAGE;
                gce._param1=Util.replaceSubString(_options._kickMsg,"%p",_name);
                _bus.sendData(gce,GameControlEvent.NAME,this);
                
                gce=new GameControlEvent();
                gce._type=GameControlEvent.KICK_USER;
                gce._param1=_id;
                _bus.sendData(gce,GameControlEvent.NAME,this);
            }
            else
            {
                GameControlEvent gce=new GameControlEvent();
                gce._type=GameControlEvent.MESSAGE;
                gce._param1=Util.replaceSubString(_options._okMsg,"%p",_name);
                _bus.sendData(gce,GameControlEvent.NAME,this);
            }
        }
        
        public boolean authenticate(String pwd)
        {
            if (pwd.equals(_password))
            {
                _auth=true;
                synchronized(_syncObj)
                {
                    _syncObj.notifyAll();
                }
                return true;
            }
            return false;
        }
        
        public boolean isGenericProducer()
        {
            return false;
        }

        public String[] produces()
        {
            String[] lt={GameControlEvent.NAME};
            return lt;
        }
    }
    
    public void deInitialize()
    {
    }

    public void start() throws Exception
    {
    }

    public void shutdown()
    {
    }

    public void reconfig(Node configNode) throws Exception
    {
        // reads configuration
        _options=new Options();
        String[][] rules={{"_addPlayerMsgType","addplayer_msgtype"},
                        {"_addPlayerMsgID","addplayer_msgid"},
                        {"_addPlayerPropertyName","addplayer_property_name"},
                        {"_renamePlayerMsgType","rename_msgtype"},
                        {"_renamePlayerMsgID","rename_msgid"},
                        {"_renamePlayerPropertyName","rename_property_name"},
                        {"_authCommand","auth_command"},
                        {"_propertyAddress","playerinfo_property_address"},
                        {"_tries","tries"},
                        {"_sleepTime","sleeptime"},
                        {"_kickMsg","msg_kick"},
                        {"_warnMsg","msg_warn"},
                        {"_okMsg","msg_ok"},
                        {"_firstMsg",",sg_first"},
                        {"readPlayerData()","players"}};
        String[] checkRules={
                        "_addPlayerMsgID _addPlayerMsgType ||",
                        "_addPlayerPropertyName",
                        "_renamePlayerMsgType _renamePlayerMsgID ||",
                        "_renamePlayerPropertyName",
                        "_authCommand",
                        "_propertyAddress",
                        "readPlayerData()"};
        XMLUtil.readXMLConfigNode(configNode,rules,checkRules,null,_options);
    }

    public void preStartup() throws Exception
    {
    }
    
    public void postStartup() throws Exception
    {
    }
    
    public class Options
    {
        public String _addPlayerMsgType=null;
        public String _addPlayerMsgID=null;
        public String _addPlayerPropertyName=null;
        public String _renamePlayerMsgType=null;
        public String _renamePlayerMsgID=null;
        public String _renamePlayerPropertyName=null;
        public String _authCommand=null;
        public int _tries=3;
        public int _sleepTime=10*1000;
        public String _propertyAddress=null;
        public String _kickMsg="%p has been kicked for using reserved name.";
        public String _warnMsg="%p, please authenticate yourself!";
        public String _okMsg="%p has been authenticated.";
        public String _firstMsg="%p, you are using a reserved name!";
        
        // Players, hashed by Name
        protected Properties _players=new Properties();
        
        public void readPlayerData(Node configNode, Properties data)
        {
            _players.clear();
            NodeList nl=configNode.getChildNodes();
            for (int i=0;i<nl.getLength();i++)
            {
                Node node=nl.item(i);
                if (node.getNodeType()==node.ELEMENT_NODE)
                {
                    if (node.getNodeName().equals("player"))
                    {
                        NamedNodeMap nnm=node.getAttributes();
                        Node attr=nnm.getNamedItem("name");
                        if (null==attr)
                            continue;
                        String name=attr.getNodeValue();
                        if (null==name)
                            continue;
                        attr=nnm.getNamedItem("password");
                        String pwd=attr.getNodeValue();
                        _players.put(name,pwd);
                    }
                }
            }
        }
    }
}
