/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker;

import java.io.*;
import java.util.StringTokenizer;

import org.w3c.dom.*;
import org.log4j.Category;

import com.oroinc.text.regex.*;

import de.hendriklipka.databus.*;
import de.hendriklipka.util.XMLUtil;

import de.hendriklipka.gcp.module.control.*;
import de.hendriklipka.gcp.event.*;

import de.hendriklipka.gcp.module.kicker.autokicker.eval.*;

/**
    automagically kicks player according to a rule set
    this enables kicking of player with:
    - rate too high / too low
    - cross colors
    - reserved names from wrong IP
    - "lame" names like "unnamed", "guest"...
    - players with a specific string (like clan name) in there name

    a player is kicked if one of the defined rules evaluates to TRUE
*/
public class AutoKicker implements DataBusComponent, DataBusProducer
{
    static Category CAT = Category.getInstance(AutoKicker.class.getName());

    protected PlayerHandler _playerHandler=null;
    protected PrintWriter _kickLog=null;

    protected String _copyright="(C) 1999-2000 by HLI, idea by MadMax [FSA]";

    protected String _startMsg="starting kicker...";
    protected String _endMsg="did i miss anyone?";

    protected int _sleepTime=60;
    protected int _preTime=5;

    protected String _kickFileName="kick.log";

    protected KickerThread _kickerThread=null;

    protected static boolean _firstLogRun=true;

    protected DataBus _bus=null;

    public void init(DataBus bus, Node configNode) throws Exception
    {
        _bus=bus;

        de.hendriklipka.gcp.module.kicker.autokicker.eval.EvalPattern.setCompiler(
            new Perl5Compiler(),
            Perl5Compiler.SINGLELINE_MASK |Perl5Compiler.READ_ONLY_MASK ,
            Perl5Compiler.SINGLELINE_MASK |Perl5Compiler.READ_ONLY_MASK|Perl5Compiler.CASE_INSENSITIVE_MASK );
        de.hendriklipka.gcp.module.kicker.autokicker.eval.EvalPattern.setMatcher(new Perl5Matcher());

        reconfig(configNode);
    }

    protected class KickerThread extends Thread implements DataBusProducer
    {
        public KickerThread()
        {
            this.setDaemon(true);
        }

        public void run()
        {
            while (true)
            {
                try
                {
                    if ((null!=_startMsg)&&(0!=_startMsg.length()))
                    {
                        GameControlEvent gce=new GameControlEvent();
                        gce._type=GameControlEvent.MESSAGE;
                        gce._param1=_startMsg;
                        _bus.sendData(gce,GameControlEvent.NAME,AutoKicker.this);
                        if (null!=_copyright)
                        {
                            gce=new GameControlEvent();
                            gce._type=GameControlEvent.MESSAGE;
                            gce._param1=_copyright;
                            _bus.sendData(gce,GameControlEvent.NAME,AutoKicker.this);
                        }
                        Object o=new Object();
                        synchronized(o)
                        {
                            try
                            {
                                o.wait(_preTime);
                            }
                            catch (InterruptedException ex) {}
                        }
                    }

                    GameControlEvent gce=new GameControlEvent();
                    gce._type=GameControlEvent.SERVER_STATUS;
                    GameControlResponse rp=(GameControlResponse)_bus.sendResponseData(gce,GameControlEvent.NAME,AutoKicker.this);
                    ServerInfo si=rp._serverStatus;
                    if (null==si)
                    {
                        CAT.error("no server status!!");
                    }
                    if (null!=si._players)
                    {
                        for (int i=0;i<si._players.length;i++)
                        {
                            PlayerInfo pi=si._players[i];
                            String id=pi.getID();

                            gce=new GameControlEvent();
                            gce._type=GameControlEvent.PLAYER_INFO_FOR_ID;
                            gce._param1=id;
                            rp=(GameControlResponse)_bus.sendResponseData(gce,GameControlEvent.NAME,AutoKicker.this);
                            if (null==rp._playerInfo)
                            {
                                CAT.error("no playerinfo for "+pi.getID());
                            }

                            pi.copy(rp._playerInfo);
                            _playerHandler.handlePlayer(pi,si);
                        }
                    }
                    else
                    {
                        CAT.debug("no players");
                    }
                    if ((null!=_endMsg)&&(0!=_endMsg.length()))
                    {
                        gce=new GameControlEvent();
                        gce._type=GameControlEvent.MESSAGE;
                        gce._param1=_endMsg;
                        _bus.sendData(gce,GameControlEvent.NAME,AutoKicker.this);
                    }
                }
                catch (Exception ex)
                {
                    CAT.error("error"+ex);
                }

                Object o=new Object();
                synchronized(o)
                {
                    try
                    {
                        o.wait(_sleepTime);
                    }
                    catch (InterruptedException ex) {}
                }
            }
        }

        public String[] produces()
        {
            String[] lt={GameControlEvent.NAME};
            return lt;
        }

        public boolean isGenericProducer()
        {
            return false;
        }
    }

    public void deInitialize()
    {
    }

    public void start() throws Exception
    {
        if (null==_kickerThread)
        {
            _kickerThread=new KickerThread();
            _kickerThread.start();
        }
    }

    public void shutdown()
    {
    }

    public void reconfig(Node configNode) throws Exception
    {
        NodeList nl=configNode.getChildNodes();
        for (int i=0;i<nl.getLength();i++)
        {
            Node node=nl.item(i);
            if (node.getNodeType()==node.ELEMENT_NODE)
            {
                if (node.getNodeName().equals("messages"))
                {
                    NodeList cnl=node.getChildNodes();
                    for (int j=0;j<cnl.getLength();j++)
                    {
                        Node subNode=cnl.item(j);
                        if ((subNode.getNodeType()==subNode.ELEMENT_NODE)&&(subNode.getNodeName().equals("start")))
                        {
                            String s=XMLUtil.getTextData(subNode);
                            if (0!=s.trim().length())
                                _startMsg=s;
                        }
                        else if ((subNode.getNodeType()==subNode.ELEMENT_NODE)&&(subNode.getNodeName().equals("end")))
                        {
                            String s=XMLUtil.getTextData(subNode);
                            if (0!=s.trim().length())
                                _endMsg=s;
                        }
                    }
                }
                else if (node.getNodeName().equals("config"))
                {
                    NamedNodeMap nnl=node.getAttributes();
                    Node attr=nnl.getNamedItem("kicklogfile");
                    if (null!=attr)
                    {
                        _kickFileName=attr.getNodeValue();
                    }
                    attr=nnl.getNamedItem("sleeptime");
                    if (null!=attr)
                    {
                        String s=attr.getNodeValue();
                        _sleepTime=1000*Integer.parseInt(s);
                    }
                    attr=nnl.getNamedItem("warntime");
                    if (null!=attr)
                    {
                        String s=attr.getNodeValue();
                        _preTime=1000*Integer.parseInt(s);
                    }
                }
                else if (node.getNodeName().equals("rules"))
                {
                    if (null!=_playerHandler)
                        _playerHandler.disconnect();
                    _playerHandler=new PlayerHandler(_bus);
                    _playerHandler.addRules(node);
                }
            }
        }

        try
        {
            if (null!=_kickLog)
                _kickLog.close();
            _kickLog=new PrintWriter(new FileWriter(_kickFileName,true));
            if (null!=_playerHandler)
                _playerHandler.setLogWriter(_kickLog);
        }
        catch (IOException ex)
        {
            CAT.error(ex);
            return;
        }
    }

    public void preStartup() throws Exception
    {
    }
    
    public void postStartup() throws Exception
    {
    }
}
