/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;

import org.log4j.Category;

import de.hendriklipka.databus.*;

import de.hendriklipka.util.*;

import de.hendriklipka.gcp.module.control.*;
import de.hendriklipka.gcp.module.kicker.autokicker.eval.EvaluateException;
import de.hendriklipka.gcp.module.kicker.autokicker.eval.ConfigException;
import de.hendriklipka.gcp.event.*;

/**
    handles the rule evaluation for this player
    - evaluate all rules
    - kick the player, if necessary
*/
public class PlayerHandler implements DataBusProducer
{
    static Category CAT = Category.getInstance(PlayerHandler.class.getName());

    protected PrintWriter _log=null;
    protected DataBus _bus=null;

    protected Rule[] _rules=null;

    public PlayerHandler(DataBus bus)
    {
        _bus=bus;
    }

    public void setLogWriter(PrintWriter wr)
    {
        _log=wr;
    }

    public void addRules(Node node) throws ConfigException
    {
        CAT.info("addRule");
        try
        {
            Vector v=new Vector();
            NodeList nl=node.getChildNodes();
            for (int i=0;i<nl.getLength();i++)
            {
                Node n=nl.item(i);
                if ((n.getNodeType()==n.ELEMENT_NODE)&&(n.getNodeName().equals("rule")))
                {
                    CAT.info("nodeName="+n.getNodeName());
                    Rule r=new Rule();
                    r.init(n);
                    v.addElement(r);
                }
            }
            _rules=new Rule[v.size()];
            for (int i=0;i<v.size();i++)
            {
                _rules[i]=(Rule)v.elementAt(i);
            }
        }
        catch (Throwable ex)
        {
            CAT.error("",ex);
        }
    }

    public void handlePlayer(PlayerInfo pi, ServerInfo si)
    {
        CAT.info("handlePlayer");
        Properties data=pi.getData();
        Properties serverData=si.getData();
        for (int i=0;i<_rules.length;i++)
        {
            Rule r=_rules[i];
            try
            {
                if (r.eval(data,serverData))
                {
                    // kick this player
                    GameControlEvent gce=new GameControlEvent();
                    gce._type=GameControlEvent.MESSAGE;
                    String kickMsg=r.getMessage();
                    kickMsg=Util.replaceSubString(kickMsg,"%p",pi.getName());
                    gce._param1=kickMsg;

                    _bus.sendData(gce,GameControlEvent.NAME,this);

                    gce=new GameControlEvent();
                    gce._type=GameControlEvent.KICK_USER;
                    gce._param1=pi.getID();
                    _bus.sendData(gce,GameControlEvent.NAME,this);

                    Date d=new Date(System.currentTimeMillis());
                    _log.println(d.toString()+kickMsg);
                    String[] sa=r.getLogValues(data);
                    String s="";
                    if (null!=sa)
                    {
                        for (int j=0;j<sa.length;j++)
                        {
                            _log.println("\t\t\t -> "+sa[j]);
                            s+="\n"+sa[j];
                        }
                    }
                    _log.flush();

                    CAT.info("kicked "+d.toString()+"\tplayer "+pi.getName()+" \t"+r.getMessage()+s);
                }
            }
            catch (Exception ex)
            {
                CAT.error("error",ex);
            }
        }
    }

    public boolean isGenericProducer()
    {
        return false;
    }

    public String[] produces()
    {
        String[] lt={GameControlEvent.NAME};
        return lt;
    }

    public void disconnect()
    {
    }
}
