/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker;

import java.util.Properties;
import java.util.Vector;
import java.util.StringTokenizer;

import org.w3c.dom.*;
import org.log4j.Category;

import de.hendriklipka.gcp.module.kicker.autokicker.eval.Predicate;
import de.hendriklipka.gcp.module.kicker.autokicker.eval.PredicateFactory;
import de.hendriklipka.gcp.module.kicker.autokicker.eval.ConfigException;
import de.hendriklipka.gcp.module.kicker.autokicker.eval.EvaluateException;

/**
    encapsulates a single rule for the auto kicker
*/
public class Rule
{
    static Category CAT = Category.getInstance(Rule.class.getName());

  protected Predicate[] _predicates=null;
  protected String[] _logValues=null;
  protected String _message=null;

  public Rule()
  {
  }

  public void init(Node node) throws ConfigException
  {
    NamedNodeMap nnl=node.getAttributes();
    Node attr=nnl.getNamedItem("msg");
    if (null==attr)
    {
      throw new ConfigException("missing attribute \"msg\"");
    }
    else
    {
      _message=attr.getNodeValue();
    }
    attr=nnl.getNamedItem("logvalues");
    if (null!=attr)
    {
      Vector v=new Vector();
      StringTokenizer st=new StringTokenizer(attr.getNodeValue(),";");
      while (st.hasMoreElements())
      {
        String s=(String)st.nextElement();
        v.addElement(s);
      }
      _logValues=new String[v.size()];
      for (int i=0;i<v.size();i++)
      {
        _logValues[i]=(String)v.elementAt(i);
      }
    }

    Vector v=new Vector();
    NodeList nl=node.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.ELEMENT_NODE)
      {
        Predicate p=PredicateFactory.createPredicate(n.getNodeName(),n);
        v.addElement(p);
      }
    }
    if (v.size()>0)
    {
      _predicates=new Predicate[v.size()];
      for (int i=0;i<v.size();i++)
      {
        _predicates[i]=(Predicate)v.elementAt(i);
      }
    }
  }

  public boolean eval(Properties data, Properties serverData) throws EvaluateException
  {
    CAT.info("eval");
    if (null!=_predicates)
    {
      for (int i=0;i<_predicates.length;i++)
      {
        try
        {
            // all predicates must evaluate to true
          if (!_predicates[i].eval(data, serverData))
            return false;
        }
        catch (Throwable ex)
        {
            CAT.error("error"+ex);
        }
      }
      return true;
    }
    throw new EvaluateException("no values in rule");
  }

  public String[] getLogValues(Properties data)
  {
    CAT.info("getLogvalues");
    if (null!=_logValues)
    {
      String[] sa=new String[_logValues.length];
      for (int i=0;i<sa.length;i++)
      {
        sa[i]=_logValues[i]+"="+data.getProperty(_logValues[i],"");
      }
      return sa;
    }
    return null;
  }

  public String getMessage()
  {
    return _message;
  }
}
