/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker.eval;

import java.util.Properties;

import org.w3c.dom.*;
import org.log4j.Category;

public class EqualsPredicate implements de.hendriklipka.gcp.module.kicker.autokicker.eval.Predicate
{
    static Category CAT = Category.getInstance(EqualsPredicate.class.getName());

  protected Field _firstField=null;
  protected Field _secondField=null;
  protected EvalPattern _secondPattern=null;
  protected List _secondList=null;
  protected boolean _ignoreCase=false;
  
  public void init(Node node) throws ConfigException
  {
    NamedNodeMap nnl=node.getAttributes();
    Node attr=nnl.getNamedItem("ignorecase");
    if (null!=attr)
    {
      String s=attr.getNodeValue();
      _ignoreCase=(s.equals("yes"));
    }

    NodeList nl=node.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.ELEMENT_NODE)
      {
        String s=n.getNodeName();
        if (s.equals("field"))
        {
          if (null==_firstField)
          {
            _firstField=new Field();
            _firstField.init(n);
          }
          else
          {
            if (null==_secondField)
            {
              if ((null!=_secondPattern)||(null!=_secondList))
              {
                throw new ConfigException("found second field, but have already found pattern|list while processing EQUALS");
              }
              _secondField=new Field();
              _secondField.init(n);
            }
          }
        }
        else if (s.equals("value"))
        {
          if (null==_firstField)
          {
            throw new ConfigException("found pattern, but no first field while processing EQUALS");
          }
          if ((null!=_secondField)||(null!=_secondList))
          {
            throw new ConfigException("found pattern, but hav already a second field|list while processing EQUALS");
          }
          _secondPattern=new EvalPattern();
          _secondPattern.init(n,_ignoreCase);
        }
        else if (s.equals("list"))
        {
          if (null==_firstField)
          {
            throw new ConfigException("found list, but no first field while processing EQUALS");
          }
          if ((null!=_secondField)||(null!=_secondPattern))
          {
            throw new ConfigException("found list, but hav already a second field|pattern while processing EQUALS");
          }
          _secondList=new List();
          _secondList.init(n,_ignoreCase);
        }
      }
    }
  }

  public boolean eval(Properties data, Properties serverData) throws EvaluateException
  {
    if (null==_firstField)
      throw new EvaluateException("EQUALS: no first field");
    String s1=_firstField.getValue(data,serverData);
    if (null!=_secondField)
    {
      String s2=_secondField.getValue(data,serverData);
      if (_ignoreCase)
        return s1.equalsIgnoreCase(s2);
       else
        return s1.equals(s2);
    }
    else if (null!=_secondPattern)
    {
      return _secondPattern.matches(s1);
    }
    else if (null!=_secondList)
    {
      Field[] fa=_secondList.getFields();
      if (null!=fa)
      {
        for (int i=0;i<fa.length;i++)
        {
          String s2=fa[i].getValue(data,serverData);
          if (_ignoreCase)
          {
            if (s1.equalsIgnoreCase(s2))
              return true;
          }
          else
          {
            if (s1.equals(s2))
              return true;
          }
        }
        EvalPattern[] epa=_secondList.getPatterns();
        if (null!=epa)
        {
          for (int i=0;i<epa.length;i++)
          {
            EvalPattern ep=epa[i];
            if (ep.matches(s1))
              return true;
          }
        }
      }
      return false;
    }
    throw new EvaluateException("EQUALS: no second field nor field");
  }
}
