/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker.eval;

import com.oroinc.text.regex.Pattern;
import com.oroinc.text.regex.PatternMatcher;
import com.oroinc.text.regex.PatternCompiler;
import com.oroinc.text.regex.MalformedPatternException;

import org.w3c.dom.*;
import org.log4j.Category;

public class EvalPattern
{
    static Category CAT = Category.getInstance(EvalPattern.class.getName());

  protected static PatternCompiler _compiler=null;
  protected static int _caseMask=0;
  protected static int _ignoreCaseMask=0;
  protected static PatternMatcher _matcher=null;
  
  protected Pattern _pattern=null;
  protected String _patternString=null;
  
  public static void setCompiler(PatternCompiler compiler,int i, int j)
  {
    _compiler=compiler;
    _caseMask=i;
    _ignoreCaseMask=j;
  }
  
  public static void setMatcher(PatternMatcher matcher)
  {
    _matcher=matcher;
  }
  
  public void init(Node node, boolean ignoreCase) throws ConfigException
  {
    String s="";
    NodeList nl=node.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.TEXT_NODE)
      {
        Text val=(Text)n;
        s+=val.getData().trim();
      }
    }
    _patternString=s.trim();;
    if (ignoreCase)
    {
      try
      {
        _pattern=_compiler.compile(_patternString,_ignoreCaseMask);
      }
      catch (MalformedPatternException ex)
      {
        throw new ConfigException("while processing pattern: "+ex.toString());
      }
    }
    else
    {
      try
      {
        _pattern=_compiler.compile(_patternString,_caseMask);
      }
      catch (MalformedPatternException ex)
      {
        throw new ConfigException("while processing pattern: "+ex.toString());
      }
    }
  }
  
  public boolean contains(String s)
  {
    if ((null!=_matcher)&&(null!=_pattern))
    {
      return _matcher.contains(s,_pattern);
    }
    return false;
  }
  
  public boolean matches(String s)
  {
    if ((null!=_matcher)&&(null!=_pattern))
    {
      return _matcher.matches(s,_pattern);
    }
    return false;
  }
  
  public String getAsString()
  {
    if (null!=_patternString)
    {
      return _patternString;
    }
    return "";
  }
}
