/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker.eval;

import java.util.Properties;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.w3c.dom.*;
import org.log4j.Category;

public class IpRangePredicate implements de.hendriklipka.gcp.module.kicker.autokicker.eval.Predicate
{
    static Category CAT = Category.getInstance(IpRangePredicate.class.getName());

  protected Field _field;
  protected byte[] _lower=null;
  protected byte[] _upper=null;
  
  public void init(Node node) throws ConfigException
  {
    NamedNodeMap nnl=node.getAttributes();
    Node attr=nnl.getNamedItem("lower");
    if (null==attr)
    {
      throw new ConfigException("missing attribute \"lower\" while processing IP_RANGE");
    }
    else
    {
      try
      {
        String s=attr.getNodeValue();
        InetAddress ia=InetAddress.getByName(s);
        _lower=ia.getAddress();
      }
      catch (UnknownHostException ex)
      {
        throw new ConfigException("can't parse Address while processing IP_RANGE: "+ex.toString());
      }
    }
    attr=nnl.getNamedItem("upper");
    if (null==attr)
    {
      throw new ConfigException("missing attribute \"upper\" while processing IP_RANGE");
    }
    else
    {
      try
      {
        String s=attr.getNodeValue();
        InetAddress ia=InetAddress.getByName(s);
        _upper=ia.getAddress();
      }
      catch (UnknownHostException ex)
      {
        throw new ConfigException("can't parse Address while processing IP_RANGE: "+ex.toString());
      }
    }

    NodeList nl=node.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.ELEMENT_NODE)
      {
        String s=n.getNodeName();
        if (s.equals("field"))
        {
          if (null==_field)
          {
            _field=new Field();
            _field.init(n);
          }
          else
          {
            throw new ConfigException("found more than one field while processing IP_RANGE");
          }
        }
      }
    }
  }

  public boolean eval(Properties data, Properties serverData) throws EvaluateException
  {
    if ((null!=_lower)&&(null!=_upper)&&(null!=_field))
    {
        // if we have an IP, then no DNS lookup is performed (according to the sources)
      try
      {
        InetAddress ia=InetAddress.getByName(_field.getValue(data,serverData));
        byte[] adr=ia.getAddress();
        for (int i=0;i<4;i++)
        {
          if (adr[i]>_lower[i])
            break;
          else if(adr[i]<_lower[i])
            return false;
        }
        for (int i=0;i<4;i++)
        {
          if (adr[i]<_upper[i])
            break;
          else if(adr[i]>_upper[i])
            return false;
        }
        return true;
      }
      catch (UnknownHostException ex)
      {
        throw new EvaluateException(ex.toString());
      }
    }
    throw new EvaluateException("IP_RANGE: no (lower bound or upper bound or field)");
  }
}
