/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker.eval;

import java.util.Properties;

import org.w3c.dom.*;
import org.log4j.Category;

public class LtPredicate implements de.hendriklipka.gcp.module.kicker.autokicker.eval.Predicate
{
    static Category CAT = Category.getInstance(LtPredicate.class.getName());

  protected Field _firstField;
  protected Field _secondField;
  protected EvalPattern _secondPattern;
  
  // ToDo : replace _secondPattern with double, because the value is fixed
  public void init(Node node) throws ConfigException
  {
    NodeList nl=node.getChildNodes();
    // read two sub predicates
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.ELEMENT_NODE)
      {
        String s=n.getNodeName();
        if (s.equals("field"))
        {
          if (null==_firstField)
          {
            _firstField=new Field();
            _firstField.init(n);
          }
          else
          {
            if (null==_secondField)
            {
              if (null!=_secondPattern)
              {
                throw new ConfigException("LT: found second field, but have already found pattern while processing LT");
              }
              _secondField=new Field();
              _secondField.init(n);
            }
          }
        }
        else if (s.equals("value"))
        {
          if (null==_firstField)
          {
            throw new ConfigException("LT: found pattern, but no first field while processing LT");
          }
          if (null!=_secondField)
          {
            throw new ConfigException("LT: found pattern, but hav already a second field while processing LT");
          }
          _secondPattern=new EvalPattern();
          _secondPattern.init(n,false);
        }
      }
    }
  }

  public boolean eval(Properties data, Properties serverData) throws EvaluateException
  {
    if (null==_firstField)
      throw new EvaluateException("LT: no first field");
    String s1=_firstField.getValue(data,serverData);
    String s2=null;
    if (null!=_secondField)
      s2=_secondField.getValue(data,serverData);
    else if (null!=_secondPattern)
      s2=_secondPattern.getAsString();
    else
      throw new EvaluateException("LT: no second field nor pattern");
    double d1=Double.valueOf(s1).doubleValue();
    double d2=Double.valueOf(s2).doubleValue();
    return d1<d2;
  }
}
