/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.kicker.autokicker.eval;

import java.util.Properties;

import org.w3c.dom.*;
import org.log4j.Category;


public class XorPredicate implements de.hendriklipka.gcp.module.kicker.autokicker.eval.Predicate
{
    static Category CAT = Category.getInstance(XorPredicate.class.getName());

  protected Predicate _firstPredicate=null;
  protected Predicate _secondPredicate=null;
  
  public void init(Node node) throws ConfigException
  {
    NodeList nl=node.getChildNodes();
    // read sub predicates
    for (int i=0;i<nl.getLength();i++)
    {
      Node n=nl.item(i);
      if (n.getNodeType()==n.ELEMENT_NODE)
      {
        Predicate p=PredicateFactory.createPredicate(n.getNodeName(),n);
        if (null==_firstPredicate)
        {
          _firstPredicate=p;
        }
        else if (null==_secondPredicate)
        {
          _secondPredicate=p;
        }
        else
        {
          throw new ConfigException("to much sub-expressions while processing XOR");
        }
      }
    }
  }

  public boolean eval(Properties data, Properties serverData) throws EvaluateException
  {
    CAT.info("eval");
    if ((null!=_firstPredicate)&&(null!=_secondPredicate))
    {
      boolean b1=_firstPredicate.eval(data,serverData);
      boolean b2=_secondPredicate.eval(data,serverData);
      return (b1!=b2);
    }
    throw new EvaluateException("XOR: no (first value or second value)");
  }
}
