/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
gcp@hendriklipka.de
*/

package de.hendriklipka.gcp.module.output;

import java.io.*;
import java.util.Date;
import java.text.DateFormat;

import org.w3c.dom.Node;
import org.log4j.Category;

import de.hendriklipka.util.*;
import de.hendriklipka.databus.*;

import de.hendriklipka.gcp.event.*;
import de.hendriklipka.gcp.exception.*;

/**
    writes a new log file for every map
    the map change is detected when a GameEvent with a configured name occurs
    the name for the output file can contain the mapname (%m) and the current time (%t). the output
    is appended to existing files
*/
public class MapLogWriter implements DataBusComponent, DataBusConsumer
{
    static Category CAT = Category.getInstance(MapLogWriter.class.getName());

    protected DataBus _bus=null;

    protected Options _options=new Options();
    protected PrintWriter _outputWriter=null;
    protected String LineFeed="\r\n";
    protected DateFormat _format=DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT);

    protected String _currentFile=null;
    protected String _currentMap=null;
        
    public MapLogWriter()
    {
    }

    public void init(DataBus bus, Node configNode) throws Exception
    {
        _bus=bus;
        LineFeed=System.getProperty("line.separator");

        configure(configNode);

        _bus.addConsumer(this,GameEvent.NAME);
    }
    
    public void configure(Node configNode) throws Exception
    {
        reconfig(configNode);
    }
    
    public void createOutputWriter(String name)
    {
        if (null!=_outputWriter)
        {
            _outputWriter.flush();
            _outputWriter.close();
        }
        
        if (null!=_currentMap&&null!=_currentFile&&null!=_options._programName)
        {
            callProgram(_currentFile,_currentMap);
        }
        
        Date d=new Date(System.currentTimeMillis());
//        String s=_format.format(d);
        String s=Long.toString(System.currentTimeMillis());
//        s=s.replace(File.separatorChar,'_');
        String fileName=Util.replaceSubString(_options._outfileName,"%m",name);
        fileName=Util.replaceSubString(fileName,"%t",s);
        _currentMap=name;
        _currentFile=fileName;
        try
        {
            _outputWriter=new PrintWriter(new FileWriter(fileName),true);
        }
        catch (Exception ex)
        {
            CAT.error(ex);
        }
    }

    protected void callProgram(String file, String map)
    {
        Thread t=new ProgramThread(file,map);
        t.start();
    }

    public synchronized void dataAvailable(de.hendriklipka.databus.DataBusEvent o) throws Exception
    {
        try
        {
            GameEvent ge=(GameEvent)(o._data);
            if (ge._type.equals(_options._mapStartType))
            {
                String mapName=ge._data.getProperty(_options._mapParam);
                createOutputWriter(mapName);
            }
            if (null!=_outputWriter)
            {
                String s=ge._line;
                while (s.endsWith("\n")||s.endsWith("\r"))
                    s=s.substring(0,s.length()-1);
                while (s.startsWith("\n")||s.startsWith("\r"))
                    s=s.substring(1);
                _outputWriter.print(s);
                _outputWriter.print(LineFeed);
                _outputWriter.flush();
            }
        }
        catch (Throwable ex)
        {
            CAT.error(ex);
        }
    }

    public void start() throws Exception
    {
    }
        
    public void shutdown() throws Exception
    {
    }
        
    public void deInitialize() throws Exception
    {
        _outputWriter=null;
    }
        
    public void reconfig(Node configNode) throws Exception
    {
        // the configuration is not active until the next map change
        String[][] rules={
                        {"_outfileName","outfile"},
                        {"_mapParam","mapparam"},
                        {"_programName","program"},
                        {"_mapStartType","mapstarttype"}};
        String[] checkRules={
                        "_outfileName",
                        "_mapParam",
                        "_mapStartType"};
        XMLUtil.readXMLConfigNode(configNode,rules,checkRules,null,_options);
    }
        
    public void preStartup() throws Exception
    {
    }
    
    public void postStartup() throws Exception
    {
    }
    
    public class Options
    {
        public String _outfileName=null;
        public String _mapParam=null;
        public String _mapStartType=null;
        public String _programName=null;
    }
    
    protected class ProgramThread extends Thread
    {
        protected String _name=null;
        protected String _map=null;
        
        public ProgramThread(String file, String map)
        {
            _name=file;
            _map=map;
        }
        
        public void run()
        {
            String name=_options._programName;
            name=Util.replaceAllSubStrings(name,"%n",_name);
            name=Util.replaceAllSubStrings(name,"%m",_map);
            try
            {
                CAT.debug("calling <"+name+">");
                Runtime.getRuntime().exec(name);
            }
            catch (Throwable ex)
            {
                CAT.error(ex);
            }        
        }
    }
}
