package de.hendriklipka.gcp.module.output;

import java.io.*;
import java.util.*;
import java.text.*;

import org.log4j.Category;

import org.w3c.dom.*;

import de.hendriklipka.util.*;

public class OutputFileWriterBase
{
    protected static Category CAT = Category.getInstance(OutputFileWriterBase.class.getName());

    protected Options _options=null;
    
    protected long _fileCount=0;
    protected long _fileSize=0;
    protected long _lines=0;
    
    protected long _lastChange=0;
    
    protected PrintWriter _writer=null;
    
    protected static final int ROTATE_NONE=0;
    protected static final int ROTATE_TIME=1;
    protected static final int ROTATE_SIZE=3;
    protected static final int ROTATE_LINES=4;
    
    protected static final int NAME_NONE=0;
    protected static final int NAME_NUMBER=1;
    protected static final int NAME_TIME=2;
    protected static final int NAME_RANDOM=3;
    
    protected static DateFormat _format=DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT);
    protected static Random _random=new Random();
    
    public void configure(Node configNode) throws Exception
    {
        String[][] rules={
                        {"_fileName","filename"},
                        {"handleRotateMode()","rotatemode"},
                        {"handleNamingMode()","namingmode"},
                        {"_lineDiff","changelines"},
                        {"_sizeDiff","changesize"},
                        {"_timeDiff","changetime"}};
        String[] checkRules={
                        "_fileName",
                        "handleRotateMode() ! handleRotateMode() handleNamingMode() && ||",
                        "handleNamingMode() ! handleNamingMode() _lineDiff _sizeDiff _timeDiff || || && ||"};
        _options=new Options();
        XMLUtil.readXMLConfigNode(configNode,rules,checkRules,null,_options);
    }
    
    public void writeLine(String s)
    {
        if (null==_writer)
        {
            createWriter();
            if (null==_writer)
            {
                CAT.error("can't create writer!");
                return;
            }
        }
        checkForRotation();
        _writer.println(s);
        _writer.flush();
        _lines++;
        _fileSize+=s.length();
    }
    
    protected void checkForRotation()
    {
        switch (_options._rotateMode)
        {
            case ROTATE_NONE:
                            break;
            case ROTATE_TIME:
                            long time=System.currentTimeMillis();
                            if ((time-_lastChange)>_options._timeDiff)
                            {
                                createWriter();
                                _lastChange=time;
                            }
                            break;
            case ROTATE_SIZE:
                            if (_fileSize>_options._sizeDiff)
                            {
                                _lines=0;
                                _fileSize=0;
                                createWriter();
                            }
                            break;
            case ROTATE_LINES:
                            if (_lines>_options._lineDiff)
                            {
                                _lines=0;
                                _fileSize=0;
                                createWriter();
                            }
                            break;
        }
    }
    
    protected void createWriter()
    {
        String fname=null;
        while (true)
        {
            fname=_options._fileName;
            String namePart="";
            switch (_options._namingMode)
            {
                case NAME_NONE:
                                break;
                case NAME_NUMBER:
                                namePart=Long.toString(_fileCount++);
                                break;
                case NAME_RANDOM:
                                namePart=String.valueOf(_random.nextInt());
                                break;
                case NAME_TIME:
                                String s=_format.format(new Date(System.currentTimeMillis()));
                                namePart=s.replace(File.separatorChar,'_');
                                break;
            }
            fname=Util.replaceSubString(fname,"%n",namePart);
            File f=new File(fname);
            // we are finished if the new file does not exist or we have no naming mode
            // in the last case the output file is overwritten (else we have an infinite loop [except on a creay, where this takes only 2 seconds :])
            if (!f.exists()||NAME_NONE==_options._namingMode)
                break;
        }
        try
        {
            if (null!=_writer)
            {
               _writer.close();
            }
            _writer=new PrintWriter(new FileWriter(fname));
        }
        catch (IOException ex)
        {
            CAT.error("while creating writer:",ex);
        }
    }
    
    public class Options
    {
        public boolean _writeTime=false;
        public String _fileName=null;
        public int _rotateMode=ROTATE_NONE;
        public int _namingMode=NAME_NONE;
        
        public long _lineDiff=0;
        public long _sizeDiff=0;
        public long _timeDiff=0;
        //protected String _timePoint=null;
        
        public void handleRotateMode(Node configNode, Properties data)
        {
            String s=XMLUtil.getTextData(configNode);
            if (s.equals("NONE"))
            {
                _rotateMode=ROTATE_NONE;
            }
            else if (s.equals("TIME"))
            {
                _rotateMode=ROTATE_TIME;
            }
            else if (s.equals("LINES"))
            {
                _rotateMode=ROTATE_LINES;
            }
            else if (s.equals("SIZE"))
            {
                _rotateMode=ROTATE_SIZE;
            }
        }
        
        public void handleNamingMode(Node configNode, Properties data)
        {
            String s=XMLUtil.getTextData(configNode);
            if (s.equals("NONE"))
            {
                _namingMode=NAME_NONE;
            }
            else if (s.equals("RANDOM"))
            {
                _namingMode=NAME_RANDOM;
            }
            else if (s.equals("NUMBER"))
            {
                _namingMode=NAME_NUMBER;
            }
            else if (s.equals("TIME"))
            {
                _namingMode=NAME_TIME;
            }
        }
    }
}
